Method: MessageBus::Implementation#after_fork

Defined in:
lib/message_bus.rb

#after_forkvoid

This method returns an undefined value.

Performs routines that are necessary after a process fork, typically

triggered by a forking webserver. Performs whatever the backend requires
and ensures the server is listening for publications and running
scheduled tasks.


560
561
562
563
564
565
# File 'lib/message_bus.rb', line 560

def after_fork
  backend_instance.after_fork
  ensure_subscriber_thread
  # will ensure timer is running
  timer.queue {}
end