Method: MessageBus::Implementation#global_backlog

Defined in:
lib/message_bus.rb

#global_backlog(last_id = nil) ⇒ Array<MessageBus::Message>

Get messages from the global backlog since the last ID specified

Parameters:

  • last_id (#to_i) (defaults to: nil)

    the global ID of the last message that the caller received

Returns:

  • (Array<MessageBus::Message>)

    all messages published on any channel since the specified last ID



475
476
477
# File 'lib/message_bus.rb', line 475

def global_backlog(last_id = nil)
  backlog(nil, last_id)
end