Method: MessageBus::Implementation#local_unsubscribe

Defined in:
lib/message_bus.rb

#local_unsubscribe(channel = nil, &blk) ⇒ void

This method returns an undefined value.

Removes a subscription to a particular channel, filtered by the current site (@see #site_id_lookup).

Parameters:

  • channel (String) (defaults to: nil)

    the name of the channel from which we should unsubscribe

  • blk (Proc, nil)

    the callback which should be removed. If nil, removes all.



465
466
467
468
# File 'lib/message_bus.rb', line 465

def local_unsubscribe(channel = nil, &blk)
  site_id = site_id_lookup.call if site_id_lookup
  unsubscribe_impl(channel, site_id, &blk)
end