Method: MessageBus::Implementation#timer

Defined in:
lib/message_bus.rb

#timerMessageBus::TimerThread

Returns the timer thread used for triggering scheduled routines at specific times/intervals.

Returns:

  • (MessageBus::TimerThread)

    the timer thread used for triggering scheduled routines at specific times/intervals.



580
581
582
583
584
585
586
587
588
589
590
# File 'lib/message_bus.rb', line 580

def timer
  return @timer_thread if @timer_thread

  @timer_thread ||= begin
    t = MessageBus::TimerThread.new
    t.on_error do |e|
      logger.warn "Failed to process job: #{e} #{e.backtrace}"
    end
    t
  end
end