Method: MessageBus::Implementation#local_subscribe
- Defined in:
- lib/message_bus.rb
#local_subscribe(channel = nil, last_id = -1,) {|message| ... } ⇒ Proc
Subscribe to messages on a particular channel, filtered by the current site (@see #site_id_lookup). Each message since the last ID specified will be delivered by yielding to the passed block as soon as it is available. This will not block, but instead the callbacks will be executed asynchronously in a dedicated subscriber thread.
443 444 445 446 |
# File 'lib/message_bus.rb', line 443 def local_subscribe(channel = nil, last_id = -1, &blk) site_id = site_id_lookup.call if site_id_lookup && !global?(channel) subscribe_impl(channel, site_id, last_id, &blk) end |